using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;
//using MyInteropCode;
using bumper = Microsoft.Robotics.Services.ContactSensor.Proxy;
using drive = Microsoft.Robotics.Services.Drive.Proxy;
using input = Microsoft.Robotics.Services.Sample.DirectionDialog.Proxy;
using brick = Microsoft.Robotics.Services.Sample.Lego.Nxt.Brick.Proxy;


namespace LegoTriBot
{
	[Contract(Contract.Identifier)]
	[DisplayName("LegoTriBot")]
	[Description("LegoTriBot service (no description provided)")]
	class LegoTriBotService : DsspServiceBase
	{
        // partnerships with bumper, differential drive
        [Partner("bumper", Contract = bumper.Contract.Identifier,
            CreationPolicy = PartnerCreationPolicy.UseExisting)]
        private
            bumper.ContactSensorArrayOperations _bumperPort = new bumper.ContactSensorArrayOperations();

        [Partner("drive", Contract = drive.Contract.Identifier, CreationPolicy = PartnerCreationPolicy.UseExisting)]
        private drive.DriveOperations _drivePort = new drive.DriveOperations();

            
        [Partner("input", Contract = input.Contract.Identifier, CreationPolicy = PartnerCreationPolicy.CreateAlways)]
        private input.DirectionDialogOperations _inputPort = new input.DirectionDialogOperations();

		[ServiceState]
		LegoTriBotState _state = new LegoTriBotState();
		[ServicePort("/LegoTriBot", AllowMultipleInstances = true)]
		LegoTriBotOperations _mainPort = new LegoTriBotOperations();
    
 		
		public LegoTriBotService(DsspServiceCreationPort creationPort)
			: base(creationPort)
		{
		}
		
		protected override void Start()
		{
			
			// 
			// Add service specific initialization here
			// 

			base.Start();

            // subscribe to button presses and thumbstick changes on the 360 pad
            input.DirectionDialogOperations inputNotify = new input.DirectionDialogOperations();
            _inputPort.Subscribe(inputNotify);
            Activate(Arbiter.Receive<input.ButtonPress>(true, inputNotify, ButtonPressHandler));

            // subscribe to bumper notifications
            bumper.ContactSensorArrayOperations bumperNotify = new bumper.ContactSensorArrayOperations();
            _bumperPort.Subscribe(bumperNotify);
            Activate(Arbiter.Receive<bumper.Update>(true, bumperNotify, BumperHandler));

 		}

        private void ButtonPressHandler(input.ButtonPress msg)
        {
            drive.SetDrivePowerRequest req = new drive.SetDrivePowerRequest();  
            switch (msg.Body.Direction)
            {
                case input.ButtonDirection.Forwards:
                    // Left/RightWheelPower expects a value from -1.0f to 1.0f.
                    req.LeftWheelPower = 0.5;
                    req.RightWheelPower = 0.5;
                    _drivePort.SetDrivePower(req);
                    break;
                case input.ButtonDirection.Backwards:
                    // Left/RightWheelPower expects a value from -1.0f to 1.0f.
                    req.LeftWheelPower = -0.5;
                    req.RightWheelPower = -0.5;
                    _drivePort.SetDrivePower(req);
                    break;
                case input.ButtonDirection.Left:
                    // Left/RightWheelPower expects a value from -1.0f to 1.0f.
                    req.LeftWheelPower = 0.2;
                    req.RightWheelPower = -0.2;
                    _drivePort.SetDrivePower(req);
                    break;
                case input.ButtonDirection.Right:
                    // Left/RightWheelPower expects a value from -1.0f to 1.0f.
                    req.LeftWheelPower = -0.2;
                    req.RightWheelPower = 0.2;
                    _drivePort.SetDrivePower(req);
                    break;
                case input.ButtonDirection.Stop:
                    // Left/RightWheelPower expects a value from -1.0f to 1.0f.
                    req.LeftWheelPower = 0.0;
                    req.RightWheelPower = 0.0;
                    _drivePort.SetDrivePower(req);
                    break;
            }
           OOPSMPManagedCodeWrapper.Class1 c = new OOPSMPManagedCodeWrapper.Class1();
           c.hello();
   
        }

        private void BumperHandler(bumper.Update msg)
        {
            if (msg.Body.Pressed)
            {
                LogInfo("Pressed!");
                drive.SetDrivePowerRequest req = new drive.SetDrivePowerRequest();
                req.LeftWheelPower = 0.0f;
                req.RightWheelPower = 0.0f;
                _drivePort.SetDrivePower(req);
            }
        }


	}
}


